@@ -1,4 +1,5 @@
-from typing import TYPE_CHECKING, Union
+from datetime import datetime
+from typing import TYPE_CHECKING, cast
 
 from asyncz.executors.base import BaseExecutor, run_task
 
@@ -15,14 +16,13 @@ class DebugExecutor(BaseExecutor):
     def do_send_task(
         self,
         task: "TaskType",
-        run_times: Union[
-            int,
-            str,
-        ],
-    ):
+        run_times: list[datetime],
+    ) -> None:
+        assert task.id is not None, "Cannot send decorator type task"
+        assert self.logger is not None, "logger is None"
         try:
-            events = run_task(task, task.store_alias, run_times, self.logger)
-        except BaseException:
+            events = run_task(task, cast(str, task.store_alias), run_times, self.logger)
+        except Exception:
             self.run_task_error(task.id)
         else:
             self.run_task_success(task.id, events)
