@@ -1,16 +1,14 @@
 from functools import partial, wraps
-from typing import Any
-
-from asyncz.typing import DictAny
+from typing import Any, Callable
 
 
-def run_in_event_loop(fn):
+def run_in_event_loop(fn: Callable[..., Any]) -> Any:
     """
-    Decorator to run in an evenmt loop.
+    Decorator to run in an event loop.
     """
 
     @wraps(fn)
-    def wrapper(self, *args: Any, **kwargs: "DictAny"):
+    def wrapper(self, *args: Any, **kwargs: Any) -> None:  # type: ignore
         wrapped = partial(fn, self, *args, **kwargs)
         self.event_loop.call_soon_threadsafe(wrapped)
 
