@@ -1,23 +1,28 @@
 import inspect
 import re
-from asyncio import iscoroutinefunction
-from calendar import timegm
 from datetime import date, datetime, time, timedelta, tzinfo
+from datetime import timezone as dttz
 from functools import partial
-from typing import Any, Dict, Union
+from typing import Any, Callable, Optional, Union, cast, overload
 
 from asyncz.exceptions import AsynczException, AsynczLookupError
-from pytz import FixedOffset, timezone, utc
 
 try:
-    from threading import TIMEOUT_MAX
+    from tzlocal import get_localzone as _get_localzone
+
+    def get_localzone() -> tzinfo:
+        return _get_localzone()
 except ImportError:
-    TIMEOUT_MAX = 4294967
+
+    def get_localzone() -> tzinfo:
+        return dttz.utc
+
+
+try:
+    from zoneinfo import ZoneInfo  # type: ignore[import-not-found,unused-ignore]
+except ImportError:
+    from backports.zoneinfo import ZoneInfo  # type: ignore[no-redef,unused-ignore]
 
-BOOL_VALIDATION = {
-    "true": ["true", "yes", "on", "y", "t", "1", True],
-    "false": ["false", "no", "off", "f", "0", False],
-}
 
 DATE_REGEX = re.compile(
     r"(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})"
@@ -27,61 +32,90 @@ DATE_REGEX = re.compile(
 )
 
 
-def repr_escape(string):
-    return string
+@overload
+def to_int(value: None) -> None: ...
 
 
-def to_int(value: Union[str, float]) -> int:
+@overload
+def to_int(value: Union[str, float]) -> int: ...
+
+
+def to_int(value: Union[str, float, None]) -> Optional[int]:
     """
     Safely converts a value to integer.
     """
     if value is not None:
         return int(value)
+    return None
 
 
-def to_float(value: Union[str, int]) -> float:
+@overload
+def to_float(value: None) -> None: ...
+
+
+@overload
+def to_float(value: Union[str, int, float]) -> float: ...
+
+
+def to_float(value: Union[str, int, float, None]) -> Optional[float]:
     """
     Safely converts a value to float.
     """
     if value is not None:
         return float(value)
+    return None
 
 
-def to_bool(value: str) -> bool:
-    """
-    Converts the given value into a boolean.
-    """
-    if isinstance(value, str):
-        value = value.strip().lower()
-    if value in BOOL_VALIDATION["true"]:
-        return True
-    elif value in BOOL_VALIDATION["false"]:
-        return False
-    return False
+@overload
+def to_timezone(value: None) -> None: ...
 
 
-def to_timezone(value: Any) -> timezone:
+@overload
+def to_timezone(value: Union[str, tzinfo]) -> tzinfo: ...
+
+
+def to_timezone(value: Any) -> Optional[tzinfo]:
     """
     Converts a value to timezone object.
     """
     if isinstance(value, str):
-        return timezone(value)
+        # python 3.8 typing issue
+        return cast(tzinfo, ZoneInfo(value))
     if isinstance(value, tzinfo):
-        if not hasattr(value, "localize") or not hasattr(value, "normalize"):
-            raise TypeError("Only timezones from the pytz library are supported.")
-        if value.tzname(None) == "local":
-            raise ValueError(
-                "Unable to determine the name of the local timezone -- you must explicitly "
-                "specify the name of the local timezone. Please refrain from using timezones like "
-                "BST to prevent problems with daylight saving time. Instead, use a locale based "
-                "timezone name (such as Europe/London)."
-            )
         return value
     if value is not None:
-        raise TypeError("Expected tzinfo, got %s instead" % value.__class__.__name__)
+        raise TypeError(f"Expected tzinfo, got {value.__class__.__name__} instead")
+    return None
 
 
-def to_datetime(value: Union[str, datetime], tz: tzinfo, arg_name: str) -> datetime:
+def to_timezone_with_fallback(value: Any = None) -> tzinfo:
+    timezone: Optional[tzinfo] = to_timezone(value)
+    if timezone is None:
+        timezone = get_localzone()
+    return timezone
+
+
+@overload
+def to_datetime(
+    value: None, tz: Union[tzinfo, str, None], arg_name: str, require_tz: bool = True
+) -> None: ...
+
+
+@overload
+def to_datetime(
+    value: Union[str, datetime, date],
+    tz: Union[tzinfo, str, None],
+    arg_name: str,
+    require_tz: bool = True,
+) -> Union[datetime, Any]: ...
+
+
+def to_datetime(
+    value: Union[str, datetime, date, None],
+    tz: Union[tzinfo, str, None],
+    arg_name: str,
+    require_tz: bool = True,
+) -> Union[datetime, None, Any]:
     """
     Converts the given value to a timezone compatible aware datetime object.
 
@@ -90,7 +124,7 @@ def to_datetime(value: Union[str, datetime], tz: tzinfo, arg_name: str) -> datet
     If the input is a string, it is parsed as a datetime with the given timezone.
     """
     if not value or value is None:
-        return
+        return None
 
     if isinstance(value, datetime):
         _datetime = value
@@ -106,14 +140,14 @@ def to_datetime(value: Union[str, datetime], tz: tzinfo, arg_name: str) -> datet
         tzname = values.pop("timezone")
 
         if tzname == "Z":
-            tz = utc
+            tz = dttz.utc
         elif tzname:
             hours, minutes = (int(x) for x in tzname[1:].split(":"))
             sign = 1 if tzname[0] == "+" else -1
-            tz = FixedOffset(sign * (hours * 60 + minutes))
+            tz = dttz(sign * timedelta(hours=hours, minutes=minutes))
 
         values = {k: int(v or 0) for k, v in values.items()}
-        _datetime = datetime(**values)
+        _datetime = datetime(**values)  # type: ignore
     else:
         raise AsynczException(
             detail=f"Unsupported type for {arg_name}: {value.__class__.__name__}."
@@ -123,34 +157,54 @@ def to_datetime(value: Union[str, datetime], tz: tzinfo, arg_name: str) -> datet
         return _datetime
 
     if tz is None:
+        if not require_tz:
+            return _datetime
         raise AsynczException(
-            detail='The "tz" argument must be specified if %s has no timezone information'
-            % arg_name
+            detail=f'The "tz" argument must be specified if {arg_name} has no timezone information'
         )
 
     if isinstance(tz, str):
-        tz = timezone(tz)
+        # python 3.8 typing issue
+        tz = cast(tzinfo, ZoneInfo(tz))
 
     return localize(_datetime, tz)
 
 
-def datetime_to_utc_timestamp(timeval: datetime):
+@overload
+def datetime_to_utc_timestamp(timeval: None) -> None: ...
+
+
+@overload
+def datetime_to_utc_timestamp(timeval: datetime) -> float: ...
+
+
+def datetime_to_utc_timestamp(timeval: Optional[datetime]) -> Optional[float]:
     """
     Converts a datetime instance to a timestamp.
     """
     if timeval is not None:
-        return timegm(timeval.utctimetuple()) + timeval.microsecond / 1000000
+        return timeval.timestamp()
+    return None
 
 
-def utc_timestamp_to_datetime(timestamp: Union[int, float]) -> datetime:
+@overload
+def utc_timestamp_to_datetime(timestamp: None) -> None: ...
+
+
+@overload
+def utc_timestamp_to_datetime(timestamp: Union[int, float]) -> datetime: ...
+
+
+def utc_timestamp_to_datetime(timestamp: Union[int, float, None]) -> Optional[datetime]:
     """
     Converts the given timestamp to a datetime instance.
     """
     if timestamp is not None:
-        return datetime.fromtimestamp(timestamp, utc)
+        return datetime.fromtimestamp(timestamp, dttz.utc)
+    return None
 
 
-def timedelta_seconds(delta: Any) -> Union[int, float]:
+def timedelta_seconds(delta: timedelta) -> float:
     """
     Converts the given timedelta to seconds.
     """
@@ -166,11 +220,11 @@ def datetime_ceil(dateval: datetime) -> datetime:
     return dateval
 
 
-def datetime_repr(dateval: datetime) -> str:
+def datetime_repr(dateval: Optional[datetime]) -> str:
     return dateval.strftime("%Y-%m-%d %H:%M:%S %Z") if dateval else "None"
 
 
-def get_callable_name(func: Any) -> str:
+def get_callable_name(func: Callable) -> str:
     """
     Returns the best available display name for the given function/callable.
     """
@@ -186,14 +240,14 @@ def get_callable_name(func: Any) -> str:
         f_class = getattr(func, "im_class", None)
 
     if f_class and hasattr(func, "__name__"):
-        return "%s.%s" % (f_class.__name__, func.__name__)
+        return f"{f_class.__name__}.{func.__name__}"
 
-    if hasattr(func, "__call__"):
+    if hasattr(func, "__call__"):  # noqa
         if hasattr(func, "__name__"):
             return func.__name__
         return func.__class__.__name__
 
-    raise TypeError("Unable to determine a name for %r -- maybe it is not a callable?" % func)
+    raise TypeError(f"Unable to determine a name for {func!r} -- maybe it is not a callable?")
 
 
 def obj_to_ref(obj: Any) -> str:
@@ -218,7 +272,7 @@ def obj_to_ref(obj: Any) -> str:
             module = obj.__module__
     else:
         module = obj.__module__
-    return "%s:%s" % (module, name)
+    return f"{module}:{name}"
 
 
 def ref_to_obj(ref: str) -> Any:
@@ -234,17 +288,25 @@ def ref_to_obj(ref: str) -> Any:
     try:
         obj = __import__(modulename, fromlist=[rest])
     except ImportError:
-        raise AsynczLookupError("Error resolving reference %s: could not import module" % ref)
-
+        raise AsynczLookupError(
+            f"Error resolving reference {ref}: could not import module"
+        ) from None
     try:
         for name in rest.split("."):
             obj = getattr(obj, name)
+
+        # It might come from a class containing the fn attribute
+        # implementation of the __call__ method
+        if hasattr(obj, "fn"):
+            obj = obj.fn
         return obj
     except Exception:
-        raise AsynczLookupError("Error resolving reference %s: error looking up object" % ref)
+        raise AsynczLookupError(
+            f"Error resolving reference {ref}: error looking up object"
+        ) from None
 
 
-def maybe_ref(ref: str) -> Any:
+def maybe_ref(ref: Any) -> Any:
     """
     Returns the object that the given reference points to, if it is indeed a reference.
     If it is not a reference, the object is returned as-is.
@@ -254,7 +316,25 @@ def maybe_ref(ref: str) -> Any:
     return ref_to_obj(ref)
 
 
-def check_callable_args(func, args: Any, kwargs: Dict[Any, Any]) -> None:
+def make_function(func: Any) -> Callable[..., Any]:
+    """For wrapping generator fns."""
+
+    def _(*args: Any, **kwargs: Any) -> Any:
+        return func(*args, **kwargs)
+
+    return _
+
+
+def make_async_function(func: Any) -> Callable[..., Any]:
+    """For wrapping generator fns."""
+
+    async def _(*args: Any, **kwargs: Any) -> Any:
+        return await func(*args, **kwargs)
+
+    return _
+
+
+def check_callable_args(func: Callable[..., Any], args: Any, kwargs: Any) -> None:
     """
     Ensures that the given callable can be called with the given arguments.
     """
@@ -269,7 +349,7 @@ def check_callable_args(func, args: Any, kwargs: Dict[Any, Any]) -> None:
     try:
         sig = inspect.signature(func, follow_wrapped=False)
     except ValueError as e:
-        raise AsynczException(detail=str(e))
+        raise AsynczException(detail=str(e)) from e
 
     for param in sig.parameters.values():
         if param.kind == param.POSITIONAL_OR_KEYWORD:
@@ -301,25 +381,30 @@ def check_callable_args(func, args: Any, kwargs: Dict[Any, Any]) -> None:
 
     if pos_kwargs_conflicts:
         raise ValueError(
-            "The following arguments are supplied in both args and kwargs: %s"
-            % ", ".join(pos_kwargs_conflicts)
+            "The following arguments are supplied in both args and kwargs: {}".format(
+                ", ".join(pos_kwargs_conflicts)
+            )
         )
 
     if positional_only_kwargs:
         raise ValueError(
-            "The following arguments cannot be given as keyword arguments: %s"
-            % ", ".join(positional_only_kwargs)
+            "The following arguments cannot be given as keyword arguments: {}".format(
+                ", ".join(positional_only_kwargs)
+            )
         )
 
     if unsatisfied_args:
         raise ValueError(
-            "The following arguments have not been supplied: %s" % ", ".join(unsatisfied_args)
+            "The following arguments have not been supplied: {}".format(
+                ", ".join(unsatisfied_args)
+            )
         )
 
     if unsatisfied_kwargs:
         raise ValueError(
-            "The following keyword-only arguments have not been supplied in kwargs: %s"
-            % ", ".join(unsatisfied_kwargs)
+            "The following keyword-only arguments have not been supplied in kwargs: {}".format(
+                ", ".join(unsatisfied_kwargs)
+            )
         )
 
     if not has_varargs and unmatched_args:
@@ -330,23 +415,20 @@ def check_callable_args(func, args: Any, kwargs: Dict[Any, Any]) -> None:
 
     if not has_var_kwargs and unmatched_kwargs:
         raise ValueError(
-            "The target callable does not accept the following keyword arguments: %s"
-            % ", ".join(unmatched_kwargs)
+            "The target callable does not accept the following keyword arguments: {}".format(
+                ", ".join(unmatched_kwargs)
+            )
         )
 
 
-def iscoroutinefunction_partial(f: Any) -> bool:
-    while isinstance(f, partial):
-        f = f.func
-    return iscoroutinefunction(f)
-
-
 def normalize(value: datetime) -> datetime:
+    # applies dst change
     return datetime.fromtimestamp(value.timestamp(), value.tzinfo)
 
 
 def localize(value: datetime, tzinfo: tzinfo) -> datetime:
     if hasattr(tzinfo, "localize"):
-        return tzinfo.localize(value)
+        # pytz localize
+        return cast(datetime, tzinfo.localize(value))
 
     return normalize(value.replace(tzinfo=tzinfo))
